#
# Copyright 2018.  Cumulus Networks, Inc.
#
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class AcctonAS7326SFP28(cumulus.platform.TridentThree25GPort):
    def __init__(self, label, serdes, rx_lane_map=None, tx_lane_map=None,
                 rx_polarity_flip=None, tx_polarity_flip=None, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=None,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp


class AcctonAS7326QSFP28(cumulus.platform.TridentThree100GPort):
    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=None,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp


class AcctonAS7326_56XSwitch(cumulus.platform.BCMSwitch):
    _sfp28 = AcctonAS7326SFP28
    _qsfp28 = AcctonAS7326QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False
    ports = (
        _sfp28(label="1",   serdes=(2,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="2",   serdes=(1,), tx_lane_map=0x3210,
              rx_lane_map=0x1230, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="3",   serdes=(3,), tx_lane_map=0x3210,
              rx_lane_map=0x1230, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="4",   serdes=(7,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="5",   serdes=(6,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True),
        _sfp28(label="6",   serdes=(0,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="7",   serdes=(4,), tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="8",  serdes=(15,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=True, rx_polarity_flip=True),
        _sfp28(label="9",   serdes=(5,), tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="10",  serdes=(13,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="11",   serdes=(12,), tx_lane_map=0x3210,
              rx_lane_map=0x3012, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="12",  serdes=(14,), tx_lane_map=0x3210,
              rx_lane_map=0x3012, tx_polarity_flip=False, rx_polarity_flip=True),
        _sfp28(label="13",  serdes=(22,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="14",  serdes=(21,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="15",  serdes=(23,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="16",  serdes=(31,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="17",  serdes=(30,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="18",  serdes=(20,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="19",  serdes=(28,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="20",  serdes=(35,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=True,  rx_polarity_flip=True,),
        _sfp28(label="21",  serdes=(29,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="22",  serdes=(33,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="23",  serdes=(32,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="24",  serdes=(34,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="25",  serdes=(42,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="26",  serdes=(41,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="27",  serdes=(43,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="28",  serdes=(51,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="29",  serdes=(50,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="30",  serdes=(40,),  tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="31",  serdes=(48,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="32",  serdes=(59,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="33",  serdes=(49,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="34",  serdes=(57,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="35",  serdes=(56,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=False,  rx_polarity_flip=True,),
        _sfp28(label="36",  serdes=(58,), tx_lane_map=0x3210,
               rx_lane_map=0x3012, tx_polarity_flip=False,  rx_polarity_flip=True,),
        _sfp28(label="37",  serdes=(61,), tx_lane_map=0x3210,
               rx_lane_map=0x2310, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="38",  serdes=(62,), tx_lane_map=0x3210,
               rx_lane_map=0x2310, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="39",  serdes=(63,), tx_lane_map=0x3210,
               rx_lane_map=0x2310, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="40",  serdes=(64,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="41",  serdes=(65,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="42",  serdes=(60,), tx_lane_map=0x3210,
               rx_lane_map=0x2310, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="43",  serdes=(67,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="44",  serdes=(68,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="45",  serdes=(66,), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="46",  serdes=(70,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="47",  serdes=(71,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="48",  serdes=(69,), tx_lane_map=0x3210,
               rx_lane_map=0x1230, tx_polarity_flip=False, rx_polarity_flip=True,),

        _qsfp28(label="49", serdes=(  76,  77,   78,   79),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x000c,
              tx_lane_map=0x3120, rx_lane_map=0x2130),
        _qsfp28(label="50", serdes=(  84,   85,   86,   87),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000c,
              tx_lane_map=0x2031, rx_lane_map=0x1203),
        _qsfp28(label="51", serdes=(92, 93, 94, 95),
              tx_polarity_flip=0x0009, rx_polarity_flip=0x000c,
              tx_lane_map=0x3210, rx_lane_map=0x2130),
        _qsfp28(label="52", serdes=( 96,  97,  98,  99),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x0003,
              tx_lane_map=0x2031, rx_lane_map=0x1203),
        _qsfp28(label="53", serdes=(  104,  105,  106,  107),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x000c,
              tx_lane_map=0x3120, rx_lane_map=0x2130),
        _qsfp28(label="54", serdes=( 112, 113, 114, 115),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000c,
              tx_lane_map=0x2031, rx_lane_map=0x1203),
        _qsfp28(label="55", serdes=( 120, 121, 122, 123),
                tx_polarity_flip=0x0007, rx_polarity_flip=0x0002,
                tx_lane_map=0x3210, rx_lane_map=0x2130),
        _qsfp28(label="56", serdes=( 124, 125, 126, 127),
                tx_polarity_flip=0x0003, rx_polarity_flip=0x000c,
                tx_lane_map=0x2031, rx_lane_map=0x1203),
    )


class AcctonAS7326_56XCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/platform/accton_as7326_56x_cpld.0'


class AcctonAS7326_56XTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Temp sensor behind networking asic'
    driver_path = '/sys/bus/i2c/devices/26-0048'
    temp_fan_min = 41
    temp_fan_max = 55
    temp_max = 61
    temp_crit = 67


class AcctonAS7326_56XTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Temp sensor in front of networking asic'
    driver_path = '/sys/bus/i2c/devices/26-0049'
    temp_fan_min = 46
    temp_fan_max = 60
    temp_max = 66
    temp_crit = 69


class AcctonAS7326_56XTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Temp sensor front left'
    driver_path = '/sys/bus/i2c/devices/26-004a'
    temp_fan_min = 32
    temp_fan_max = 50
    temp_max = 56
    temp_crit = 59


class AcctonAS7326_56XTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    driver_path = '/sys/bus/i2c/devices/26-004b'
    description = 'Temp Sensor Near CPU'
    temp_fan_min = 33
    temp_fan_max = 50
    temp_max = 57
    temp_crit = 60


AcctonAS7326_56X_core_path = '/sys/devices/platform/coretemp.0'


class AcctonAS7326_56XCPUTemp(cumulus.sensors.Temp_Unit):
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS7326_56X_core_path
    temp_fan_min = 58
    temp_fan_max = 65
    temp_max = 70
    temp_crit = 98


class AcctonAS7326_56XTemp5(AcctonAS7326_56XCPUTemp):
    name = 'Temp5'
    driver_hwmon = ['temp2']


class AcctonAS7326_56XTemp6(AcctonAS7326_56XCPUTemp):
    name = 'Temp6'
    driver_hwmon = ['temp3']


class AcctonAS7326_56XTemp7(AcctonAS7326_56XCPUTemp):
    name = 'Temp7'
    driver_hwmon = ['temp4']


class AcctonAS7326_56XTemp8(AcctonAS7326_56XCPUTemp):
    name = 'Temp8'
    driver_path = AcctonAS7326_56X_core_path
    driver_hwmon = ['temp5']


AcctonAS7326_56X_PSU1_i2c_path = '/sys/bus/i2c/devices/30-0059'
AcctonAS7326_56X_PSU2_i2c_path = '/sys/bus/i2c/devices/24-005b'


class AcctonAS7326_56XPSUTemp(cumulus.sensors.Temp_Unit):
    temp_crit = 70
    temp_max = 60
    temp_fan_min = 25
    psu = None
    cpld_path = AcctonAS7326_56XCPLD().path

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.cpld_psu).find('ok') >= 0:
            return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)
        else:
            return -1

class AcctonAS7326_56XTemp9(AcctonAS7326_56XPSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS7326_56X_PSU1_i2c_path
    driver_hwmon = ['temp1']
    cpld_psu = 'psu_pwr1'


class AcctonAS7326_56XTemp10(AcctonAS7326_56XPSUTemp):
    psu = 'PSU1'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS7326_56X_PSU1_i2c_path
    driver_hwmon = ['temp2']
    cpld_psu = 'psu_pwr1'


class AcctonAS7326_56XTemp11(AcctonAS7326_56XPSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp1'
    description = psu + ' Inlet Temp Sensor'
    driver_path = AcctonAS7326_56X_PSU2_i2c_path
    driver_hwmon = ['temp1']
    cpld_psu = 'psu_pwr2'


class AcctonAS7326_56XTemp12(AcctonAS7326_56XPSUTemp):
    psu = 'PSU2'
    name = psu + 'Temp2'
    description = psu + ' Max Temp Sensor'
    driver_path = AcctonAS7326_56X_PSU2_i2c_path
    driver_hwmon = ['temp2']
    cpld_psu = 'psu_pwr2'


class AcctonAS7326_56X_ASICTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = '/cumulus/switchd/run'
    temp_fan_min = 67
    temp_fan_max = 82
    temp_max = 87
    temp_crit = 95


class AcctonAS7326_56XTemp13(AcctonAS7326_56X_ASICTemp):
    name = 'Temp9'
    driver_hwmon = ['temp1']


class AcctonAS7326_56XTemp14(AcctonAS7326_56X_ASICTemp):
    name = 'Temp10'
    driver_hwmon = ['temp2']


class AcctonAS7326_56XTemp15(AcctonAS7326_56X_ASICTemp):
    name = 'Temp11'
    driver_hwmon = ['temp3']


class AcctonAS7326_56XTemp16(AcctonAS7326_56X_ASICTemp):
    name = 'Temp12'
    driver_hwmon = ['temp4']


class AcctonAS7326_56XTemp17(AcctonAS7326_56X_ASICTemp):
    name = 'Temp13'
    driver_hwmon = ['temp5']


class AcctonAS7326_56XTemp18(AcctonAS7326_56X_ASICTemp):
    name = 'Temp14'
    driver_hwmon = ['temp6']


class AcctonAS7326_56XTemp19(AcctonAS7326_56X_ASICTemp):
    name = 'Temp15'
    driver_hwmon = ['temp7']


class AcctonAS7326_56XTemp20(AcctonAS7326_56X_ASICTemp):
    name = 'Temp16'
    driver_hwmon = ['temp8']


class AcctonAS7326_56X_PSU(cumulus.sensors.PSU_Unit):
    init_done = False

    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod', 'cpr4011']
            retrm = subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'cpr4011']
                subprocess.call(cmd)
            self.init_done = True


class AcctonAS7326_56XPSU1(AcctonAS7326_56X_PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']


class AcctonAS7326_56XPSU2(AcctonAS7326_56X_PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']


class AcctonAS7326_56XFanBase(cumulus.sensors.Fan_Unit):
    hwmon_search = False
    driver_path = AcctonAS7326_56XCPLD.path
    # The minimum value that the
    # pwm register can hold is 4. Mapping
    # that to 0~255 range by multiplying
    # by 17.
    minpwm = 119
    # The CPLD driver only exports "pwm1", which controls all the
    # fans.
    pwm_hwmon = ['pwm1']


# 7326 has 6 fan trays, each of which has a front
# and a rear fan.
class AcctonAS7326_56XFan(AcctonAS7326_56XFanBase):
    fan_min = 13500
    fan_max = 23300


class AcctonAS7326_56XFanR(AcctonAS7326_56XFanBase):
    fan_min = 12600
    fan_max = 19600


class AcctonAS7326_56XFan1(AcctonAS7326_56XFan):
    name = 'Fan1'
    description = 'Fan Tray 1, Fan A'
    # driver is fan speed ...
    driver_hwmon = ['fan1']
    # status is "cpld_hwmon"
    cpld_hwmon = ['fan_1']


class AcctonAS7326_56XFan2(AcctonAS7326_56XFanR):
    name = 'Fan2'
    description = 'Fan Tray 1, Fan B'
    driver_hwmon = ['fan2']
    cpld_hwmon = ['fan_1']


class AcctonAS7326_56XFan3(AcctonAS7326_56XFan):
    name = 'Fan3'
    description = 'Fan Tray 2, Fan A'
    driver_hwmon = ['fan3']
    cpld_hwmon = ['fan_2']


class AcctonAS7326_56XFan4(AcctonAS7326_56XFanR):
    name = 'Fan4'
    description = 'Fan Tray 2, Fan B'
    driver_hwmon = ['fan4']
    cpld_hwmon = ['fan_2']


class AcctonAS7326_56XFan5(AcctonAS7326_56XFan):
    name = 'Fan5'
    description = 'Fan Tray 3, Fan A'
    driver_hwmon = ['fan5']
    cpld_hwmon = ['fan_3']


class AcctonAS7326_56XFan6(AcctonAS7326_56XFanR):
    name = 'Fan6'
    description = 'Fan Tray 3, Fan B'
    driver_hwmon = ['fan6']
    cpld_hwmon = ['fan_3']


class AcctonAS7326_56XFan7(AcctonAS7326_56XFan):
    name = 'Fan7'
    description = 'Fan Tray 4, Fan A'
    driver_hwmon = ['fan7']
    cpld_hwmon = ['fan_4']


class AcctonAS7326_56XFan8(AcctonAS7326_56XFanR):
    name = 'Fan8'
    description = 'Fan Tray 4, Fan B'
    driver_hwmon = ['fan8']
    cpld_hwmon = ['fan_4']


class AcctonAS7326_56XFan9(AcctonAS7326_56XFan):
    name = 'Fan9'
    description = 'Fan Tray 5, Fan A'
    driver_hwmon = ['fan9']
    cpld_hwmon = ['fan_5']


class AcctonAS7326_56XFan10(AcctonAS7326_56XFanR):
    name = 'Fan10'
    description = 'Fan Tray 5, Fan B'
    driver_hwmon = ['fan10']
    cpld_hwmon = ['fan_5']


class AcctonAS7326_56XFan11(AcctonAS7326_56XFan):
    name = 'Fan11'
    description = 'Fan Tray 6, Fan A'
    driver_hwmon = ['fan11']
    cpld_hwmon = ['fan_6']


class AcctonAS7326_56XFan12(AcctonAS7326_56XFanR):
    name = 'Fan12'
    description = 'Fan Tray 6, Fan B'
    driver_hwmon = ['fan12']
    cpld_hwmon = ['fan_6']

class AcctonAS7326_56XPSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_path = AcctonAS7326_56XCPLD().path
    fan_max = 22000
    fan_min = 6000

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.cpld_hwmon[0]).find('ok') >= 0:
            return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
        else:
            return -1

class AcctonAS7326_56XFan13(AcctonAS7326_56XPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = AcctonAS7326_56X_PSU1_i2c_path
    cpld_hwmon = ['psu_pwr1']

class AcctonAS7326_56XFan14(AcctonAS7326_56XPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = AcctonAS7326_56X_PSU2_i2c_path
    cpld_hwmon = ['psu_pwr2']


class AcctonAS7326SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'Diag'
    led_location = 'led_diag'
    bad_led_color = 'red'


class AcctonAS7326StatusLEDs():
    units = (AcctonAS7326SystemLED(),
             )


class AcctonAS7326_56XSensors():
    units = (AcctonAS7326_56XTemp1(),
             AcctonAS7326_56XTemp2(),
             AcctonAS7326_56XTemp3(),
             AcctonAS7326_56XTemp4(),
             AcctonAS7326_56XTemp5(),
             AcctonAS7326_56XTemp6(),
             AcctonAS7326_56XTemp7(),
             AcctonAS7326_56XTemp8(),
             AcctonAS7326_56XTemp9(),
             AcctonAS7326_56XTemp10(),
             AcctonAS7326_56XTemp11(),
             AcctonAS7326_56XTemp12(),
             AcctonAS7326_56XTemp13(),
             AcctonAS7326_56XTemp14(),
             AcctonAS7326_56XTemp15(),
             AcctonAS7326_56XTemp16(),
             AcctonAS7326_56XTemp17(),
             AcctonAS7326_56XTemp18(),
             AcctonAS7326_56XTemp19(),
             AcctonAS7326_56XTemp20(),
             AcctonAS7326_56XPSU1(),
             AcctonAS7326_56XPSU2(),
             AcctonAS7326_56XFan1(),
             AcctonAS7326_56XFan2(),
             AcctonAS7326_56XFan3(),
             AcctonAS7326_56XFan4(),
             AcctonAS7326_56XFan5(),
             AcctonAS7326_56XFan6(),
             AcctonAS7326_56XFan7(),
             AcctonAS7326_56XFan8(),
             AcctonAS7326_56XFan9(),
             AcctonAS7326_56XFan10(),
             AcctonAS7326_56XFan11(),
             AcctonAS7326_56XFan12(),
             AcctonAS7326_56XFan13(),
             AcctonAS7326_56XFan14(),
             )


class AcctonAS7326_56XPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (AcctonAS7326_56XFan1(),
            AcctonAS7326_56XFan2(),
            AcctonAS7326_56XFan3(),
            AcctonAS7326_56XFan4(),
            AcctonAS7326_56XFan5(),
            AcctonAS7326_56XFan6(),
            AcctonAS7326_56XFan7(),
            AcctonAS7326_56XFan8(),
            AcctonAS7326_56XFan9(),
            AcctonAS7326_56XFan10(),
            AcctonAS7326_56XFan11(),
            AcctonAS7326_56XFan12(),
            AcctonAS7326_56XFan13(),
            AcctonAS7326_56XFan14(),
            )
    temps = (AcctonAS7326_56XTemp1(),
             AcctonAS7326_56XTemp2(),
             AcctonAS7326_56XTemp3(),
             AcctonAS7326_56XTemp4(),
             AcctonAS7326_56XTemp5(),
             AcctonAS7326_56XTemp6(),
             AcctonAS7326_56XTemp7(),
             AcctonAS7326_56XTemp8(),
             AcctonAS7326_56XTemp9(),
             AcctonAS7326_56XTemp10(),
             AcctonAS7326_56XTemp11(),
             AcctonAS7326_56XTemp12(),
             AcctonAS7326_56XTemp13(),
             AcctonAS7326_56XTemp14(),
             AcctonAS7326_56XTemp15(),
             AcctonAS7326_56XTemp16(),
             AcctonAS7326_56XTemp17(),
             AcctonAS7326_56XTemp18(),
             AcctonAS7326_56XTemp19(),
             AcctonAS7326_56XTemp20(),
             )


class AcctonAS7326_56XPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (
        AcctonAS7326_56XFan13(),
    )
    temps = (
        AcctonAS7326_56XTemp9(),
        AcctonAS7326_56XTemp10(),
    )

class AcctonAS7326_56XPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (
        AcctonAS7326_56XFan14(),
    )
    temps = (
        AcctonAS7326_56XTemp11(),
        AcctonAS7326_56XTemp12(),
    )

class AcctonAS7326_56XPWMs():
    units = (
        AcctonAS7326_56XPWM1(),
        AcctonAS7326_56XPWM2(),
        AcctonAS7326_56XPWM3(),
    )


class AcctonAS7326_56X(cumulus.platform.Platform):
    name = 'accton,as7326_56x'
    chip = cumulus.platform.TridentThreeX7_56873_Chip()
    switch = AcctonAS7326_56XSwitch(chip)
    cpld = AcctonAS7326_56XCPLD()
    statusleds = AcctonAS7326StatusLEDs()
    pwms = AcctonAS7326_56XPWMs()
    sensors = AcctonAS7326_56XSensors()


Platform = AcctonAS7326_56X
