#
# Copyright (c) 2018 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.dellemc_s52xx_sensors import *

class DellS5248FSFP28(cumulus.platform.TridentThree25GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  leds=None,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  tx_lane_map=tx_lane_map,
                                                  rx_lane_map=rx_lane_map,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  unit=0,
                                                  oversub=True)
        self.sfp = sfp

class DellS5248FQSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   unit=0,
                                                   oversub=True)
        self.sfp = sfp

# Dell S5248F switch ports

class DellS5248FSwitch(cumulus.platform.BCMSwitch):
    _sfp28 = DellS5248FSFP28
    _qsfp28 = DellS5248FQSFP28
#    _qsfpdd = DellS5248FQSFP28

    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = False

    ports = (
        # TSCF12
        _sfp28(label="1",  serdes=( 48,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="2",  serdes=( 49,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="3",  serdes=( 50,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="4",  serdes=( 51,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF14
        _sfp28(label="5",  serdes=( 56,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="6",  serdes=( 57,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="7",  serdes=( 58,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="8",  serdes=( 59,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF15
        _sfp28(label="9",  serdes=( 60,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="10", serdes=( 61,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="11", serdes=( 62,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="12", serdes=( 63,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF19
        _sfp28(label="13", serdes=( 76,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="14", serdes=( 77,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="15", serdes=( 78,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="16", serdes=( 79,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF21
        _sfp28(label="17", serdes=( 84,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="18", serdes=( 85,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="19", serdes=( 86,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="20", serdes=( 87,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF23
        _sfp28(label="21", serdes=( 92,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="22", serdes=( 93,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="23", serdes=( 94,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="24", serdes=( 95,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF3
        _sfp28(label="25", serdes=( 12,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="26", serdes=( 13,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="27", serdes=( 14,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="28", serdes=( 15,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF5
        _sfp28(label="29", serdes=( 20,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="30", serdes=( 21,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="31", serdes=( 22,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="32", serdes=( 23,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF7
        _sfp28(label="33", serdes=( 28,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="34", serdes=( 29,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="35", serdes=( 30,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="36", serdes=( 31,), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=True,  rx_polarity_flip=False),

        # TSCF24
        _sfp28(label="37", serdes=( 96,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="38", serdes=( 97,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="39", serdes=( 98,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True ),
        _sfp28(label="40", serdes=( 99,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=False),

        # TSCF26
        _sfp28(label="41", serdes=(104,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="42", serdes=(105,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="43", serdes=(106,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="44", serdes=(107,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF28
        _sfp28(label="45", serdes=(112,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="46", serdes=(113,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),
        _sfp28(label="47", serdes=(114,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False),
        _sfp28(label="48", serdes=(115,), tx_lane_map=0x3210, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True ),

        # TSCF21
        #_qsfpdd(label="49", serdes=( 84,  85,  86,  87), tx_lane_map=0x1302, rx_lane_map=0x1032, tx_polarity_flip=0x1, rx_polarity_flip=0xd),

        # TSCF5
        #_qsfpdd(label="50", serdes=( 20,  21,  22,  23), tx_lane_map=0x0123, rx_lane_map=0x1032, tx_polarity_flip=0x1, rx_polarity_flip=0x8),

        # TSCF0
        _qsfp28(label="53", serdes=(  0,   1,   2,   3), tx_lane_map=0x0123, rx_lane_map=0x1302, tx_polarity_flip=0x4, rx_polarity_flip=0xe),

        # TSCF8
        _qsfp28(label="54", serdes=( 32,  33,  34,  35), tx_lane_map=0x1302, rx_lane_map=0x1032, tx_polarity_flip=0x7, rx_polarity_flip=0x2),

        # TSCF1
        _qsfp28(label="55", serdes=(  4,   5,   6,   7), tx_lane_map=0x1032, rx_lane_map=0x1302, tx_polarity_flip=0xe, rx_polarity_flip=0x1),

        # TSCF10
        _qsfp28(label="56", serdes=( 40,  41,  42,  43), tx_lane_map=0x3120, rx_lane_map=0x1032, tx_polarity_flip=0x6, rx_polarity_flip=0x2),
)

class DellS5248F(cumulus.platform.Platform):
    name = 'dellemc,s5248f_c2538'
    chip = cumulus.platform.TridentThreeX7_56873_Chip()
    switch = DellS5248FSwitch(chip)
    cpld = None
    bmc = DellS52xxBMC()
    sensors = DellS52xxSensors()
    statusleds = DellS52xxLEDs()
    pwms = None

Platform = DellS5248F
