#
# Copyright (c) 2018 Cumulus Networks, Inc.  All rights reserved.
#

import os
import subprocess
import cumulus.platform
import cumulus.ledup
import cumulus.sunit
import cumulus.sensors

vhwmon_path = '/sys/class/misc/vhwmon'

# PSUs
class DellS52xxPSU(cumulus.sensors.PSU_Unit):
    driver_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    psu_ok = 0

class DellS52xxPSU1(DellS52xxPSU):
    name = 'PSU1'
    description = 'PSU1'
    driver_hwmon = ['psu_pwr1']

    def get_status(self):
        val = self.attrs.get('psu_pwr1_all_ok', 0)
        self.attrs['psu_pwr1_all_ok'] = val
        DellS52xxPSU1.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)

class DellS52xxPSU2(DellS52xxPSU):
    name = 'PSU2'
    description = 'PSU2'
    driver_hwmon = ['psu_pwr2']

    def get_status(self):
        val = self.attrs.get('psu_pwr2_all_ok', 0)
        self.attrs['psu_pwr2_all_ok'] = val
        DellS52xxPSU2.psu_ok = int(val)
        return cumulus.sensors.PSU_Unit.get_status(self)

# Intel Denverton CPU
class DellS52xxCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

class DellS52xxTemp1(DellS52xxCoreTemp):
    name = 'Temp1'
    description = 'Physical id 0'
    driver_hwmon = ['temp1']

class DellS52xxTemp2(DellS52xxCoreTemp):
    name = 'Temp2'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class DellS52xxTemp3(DellS52xxCoreTemp):
    name = 'Temp3'
    description = 'Core 6'
    driver_hwmon = ['temp8']

class DellS52xxTemp4(DellS52xxCoreTemp):
    name = 'Temp4'
    description = 'Core 8'
    driver_hwmon = ['temp10']

class DellS52xxTemp5(DellS52xxCoreTemp):
    name = 'Temp5'
    description = 'Core 12'
    driver_hwmon = ['temp14']

# Trident3 - Internal Temp Sensors
# *** TBD ***

# BMC Sensors
class DellS52xxBMCTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + '/bmc:temp'

class DellS52xxTemp6(DellS52xxBMCTemp):
    name = 'Temp6'
    description = 'Temp_CPU'
    driver_hwmon = ['temp1']
    
class DellS52xxTemp7(DellS52xxBMCTemp):
    name = 'Temp7'
    description = 'Temp_DIMM1'
    driver_hwmon = ['temp2']
    
class DellS52xxTemp8(DellS52xxBMCTemp):
    name = 'Temp8'
    description = 'Temp_DIMM2'
    driver_hwmon = ['temp3']

class DellS52xxTemp9(DellS52xxBMCTemp):
    name = 'Temp9'
    description = 'Temp_1'
    driver_hwmon = ['temp4']
    
class DellS52xxTemp10(DellS52xxBMCTemp):
    name = 'Temp10'
    description = 'Temp_2'
    driver_hwmon = ['temp5']
    
class DellS52xxTemp11(DellS52xxBMCTemp):
    name = 'Temp11'
    description = 'Temp_3'
    driver_hwmon = ['temp6']
    
class DellS52xxTemp12(DellS52xxBMCTemp):
    name = 'Temp12'
    description = 'Temp_4'
    driver_hwmon = ['temp7']
    
class DellS52xxTemp13(DellS52xxBMCTemp):
    name = 'Temp13'
    description = 'Temp_5'
    driver_hwmon = ['temp8']
    
class DellS52xxTemp14(DellS52xxBMCTemp):
    name = 'Temp14'
    description = 'Temp_6'
    driver_hwmon = ['temp9']
    
class DellS52xxTemp15(DellS52xxBMCTemp):
    name = 'Temp15'
    description = 'Temp_7'
    driver_hwmon = ['temp10']

# PSU temp sensors
class DellS52xxPSUTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']

    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Temp_Unit.detect_state_plat(self)

class DellS52xxTemp16(DellS52xxPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp1'
    psu = DellS52xxPSU1()
    driver_path = vhwmon_path + '/bmc:psu1'

class DellS52xxTemp17(DellS52xxPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp1'
    psu = DellS52xxPSU2()
    driver_path = vhwmon_path + '/bmc:psu2'

# Fans
class DellS52xxFan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + '/bmc:fan'
    no_pwm = True

class DellS52xxFan1(DellS52xxFan):
    name = 'Fan1'
    description = 'Fan 1_1'
    driver_hwmon = ['fan5']

class DellS52xxFan2(DellS52xxFan):
    name = 'Fan2'
    description = 'Fan 1_2'
    driver_hwmon = ['fan6']

class DellS52xxFan3(DellS52xxFan):
    name = 'Fan3'
    description = 'Fan 2_1'
    driver_hwmon = ['fan7']

class DellS52xxFan4(DellS52xxFan):
    name = 'Fan4'
    description = 'Fan 2_2'
    driver_hwmon = ['fan8']

class DellS52xxFan5(DellS52xxFan):
    name = 'Fan5'
    description = 'Fan 3_1'
    driver_hwmon = ['fan9']

class DellS52xxFan6(DellS52xxFan):
    name = 'Fan6'
    description = 'Fan 3_2'
    driver_hwmon = ['fan10']

class DellS52xxFan7(DellS52xxFan):
    name = 'Fan7'
    description = 'Fan 4_1'
    driver_hwmon = ['fan11']

class DellS52xxFan8(DellS52xxFan):
    name = 'Fan8'
    description = 'Fan 4_2'
    driver_hwmon = ['fan12']

# PSU fans
class DellS52xxPSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def detect_state_plat(self):
        if not self.psu.psu_ok:
            return 'ABSENT'
        return cumulus.sensors.Fan_Unit.detect_state_plat(self)

class DellS52xxFan9(DellS52xxPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan1'
    psu = DellS52xxPSU1()
    driver_path = vhwmon_path + '/bmc:psu1'

class DellS52xxFan10(DellS52xxPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan1'
    psu = DellS52xxPSU2()
    driver_path = vhwmon_path + '/bmc:psu2'

class DellS52xxSensors():
    units = (DellS52xxPSU1(),
             DellS52xxPSU2(),
             DellS52xxTemp1(),
             DellS52xxTemp2(),
             DellS52xxTemp3(),
             DellS52xxTemp4(),
             DellS52xxTemp5(),
             DellS52xxTemp6(),
             DellS52xxTemp7(),
             DellS52xxTemp8(),
             DellS52xxTemp9(),
             DellS52xxTemp10(),
             DellS52xxTemp11(),
             DellS52xxTemp12(),
             DellS52xxTemp13(),
             DellS52xxTemp14(),
             DellS52xxTemp15(),
             DellS52xxTemp16(),
             DellS52xxTemp17(),
             DellS52xxFan1(),
             DellS52xxFan2(),
             DellS52xxFan3(),
             DellS52xxFan4(),
             DellS52xxFan5(),
             DellS52xxFan6(),
             DellS52xxFan7(),
             DellS52xxFan8(),
             DellS52xxFan9(),
             DellS52xxFan10(),
             )

class DellS52xxSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    led_location = 'led_system'
    good_led_color = 'green'
    bad_led_color = 'amber'

class DellS52xxPowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_path = '/sys/devices/pci0000:00/0000:00:0c.0/0000:04:00.0'
    led_location = 'led_power'
    good_led_color = 'green'
    bad_led_color = 'amber_blinking'

class DellS52xxLEDs():
    units = ( DellS52xxSystemLED(),
              DellS52xxPowerLED(),
              )

class DellS52xxBMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

