#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx

class MlxSN2700QSFP28(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; MlxSN2700QSFP28
;;; Two bits per port: yellow(active high), green(active high)
'''
    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2700Switch(cumulus.platform.SpectrumSwitch):
    _sfp = None
    _qsfp = MlxSN2700QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp(label="1",  adj_label=None, phy_addrs=(0,0,0,0),     hw_ports=(61,62,63,64)),
        _qsfp(label="2",  adj_label="1",  phy_addrs=(1,1,1,1),     hw_ports=(63,64), blocked_adj_split=4),
        _qsfp(label="3",  adj_label=None, phy_addrs=(2,2,2,2),     hw_ports=(57,58,59,60)),
        _qsfp(label="4",  adj_label="3",  phy_addrs=(3,3,3,3),     hw_ports=(59,60), blocked_adj_split=4),
        _qsfp(label="5",  adj_label=None, phy_addrs=(4,4,4,4),     hw_ports=(53,54,55,56)),
        _qsfp(label="6",  adj_label="5",  phy_addrs=(5,5,5,5),     hw_ports=(55,56), blocked_adj_split=4),
        _qsfp(label="7",  adj_label=None, phy_addrs=(6,6,6,6),     hw_ports=(49,50,51,52)),
        _qsfp(label="8",  adj_label="7",  phy_addrs=(7,7,7,7),     hw_ports=(51,52), blocked_adj_split=4),
        _qsfp(label="9",  adj_label=None, phy_addrs=(8,8,8,8),     hw_ports=(45,46,47,48)),
        _qsfp(label="10", adj_label="9",  phy_addrs=(9,9,9,9),     hw_ports=(47,48), blocked_adj_split=4),
        _qsfp(label="11", adj_label=None, phy_addrs=(10,10,10,10), hw_ports=(41,42,43,44)),
        _qsfp(label="12", adj_label="11", phy_addrs=(11,11,11,11), hw_ports=(43,44), blocked_adj_split=4),
        _qsfp(label="13", adj_label=None, phy_addrs=(12,12,12,12), hw_ports=(37,38,39,40)),
        _qsfp(label="14", adj_label="13", phy_addrs=(13,13,13,13), hw_ports=(39,40), blocked_adj_split=4),
        _qsfp(label="15", adj_label=None, phy_addrs=(14,14,14,14), hw_ports=(33,34,35,36)),
        _qsfp(label="16", adj_label="15", phy_addrs=(15,15,15,15), hw_ports=(35,36), blocked_adj_split=4),
        _qsfp(label="17", adj_label=None, phy_addrs=(16,16,16,16), hw_ports=(1,2,3,4)),
        _qsfp(label="18", adj_label="17", phy_addrs=(17,17,17,17), hw_ports=(3,4), blocked_adj_split=4),
        _qsfp(label="19", adj_label=None, phy_addrs=(18,18,18,18), hw_ports=(5,6,7,8)),
        _qsfp(label="20", adj_label="19", phy_addrs=(19,19,19,19), hw_ports=(7,8), blocked_adj_split=4),

        _qsfp(label="21", adj_label=None, phy_addrs=(20,20,20,20), hw_ports=(9,10,11,12)),
        _qsfp(label="22", adj_label="21", phy_addrs=(21,21,21,21), hw_ports=(11,12), blocked_adj_split=4),
        _qsfp(label="23", adj_label=None, phy_addrs=(22,22,22,22), hw_ports=(13,14,15,16)),
        _qsfp(label="24", adj_label="23", phy_addrs=(23,23,23,23), hw_ports=(15,16), blocked_adj_split=4),
        _qsfp(label="25", adj_label=None, phy_addrs=(24,24,24,24), hw_ports=(17,18,19,20)),
        _qsfp(label="26", adj_label="25", phy_addrs=(25,25,25,25), hw_ports=(19,20), blocked_adj_split=4),
        _qsfp(label="27", adj_label=None, phy_addrs=(26,26,26,26), hw_ports=(21,22,23,24)),
        _qsfp(label="28", adj_label="27", phy_addrs=(27,27,27,27), hw_ports=(23,24), blocked_adj_split=4),
        _qsfp(label="29", adj_label=None, phy_addrs=(28,28,28,28), hw_ports=(25,26,27,28)),
        _qsfp(label="30", adj_label="29", phy_addrs=(29,29,29,29), hw_ports=(27,28), blocked_adj_split=4),
        _qsfp(label="31", adj_label=None, phy_addrs=(30,30,30,30), hw_ports=(29,30,31,32)),
        _qsfp(label="32", adj_label="31", phy_addrs=(31,31,31,31), hw_ports=(31,32), blocked_adj_split=4),
    )

class MlxSN2700(cumulus.platform.Platform):
    name = 'mlnx,x86_MSN2700'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN2700Switch(chip)
    cpld = cumulus.platforms.mlnx_SN2xxx.Mlx2xxxCPLD()
    sensors = cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxSensors()
    pwms = cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxPWMs()
    statusleds = cumulus.platforms.mlnx_SN2xxx.MlxSN2xxxLEDs()

Platform = MlxSN2700
