#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class MlxSX1400SFPp(cumulus.platform.SwitchXTwo10GPort):
    _set_led = '''
;;; MlxSX1400SFPp
'''
    def __init__(self, label, adj_label, phy_addr, hw_port):
        hw_port_name = 'xe%d' % hw_port

        cumulus.platform.SwitchXTwo10GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_port=hw_port_name,
                        phy_addr=phy_addr)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_port, 'sfp+')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSX1400QSFPp(cumulus.platform.SwitchXTwo40GPort):
    _set_led = '''
;;; MlxSX1400QSFPp
;;; Two bits per port: yellow(active high), green(active high)
'''
    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.SwitchXTwo40GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp+')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSX1400LED(cumulus.ledup.LEDController):
    pass

class MlxSX1400Switch(cumulus.platform.SwitchXTwoSwitch):
    ledup = MlxSX1400LED
    _sfp = MlxSX1400SFPp
    _qsfp = MlxSX1400QSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_port(s) corresponds to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _sfp(label="1",  adj_label=None,  phy_addr=0,  hw_port=53),
        _sfp(label="2",  adj_label=None,  phy_addr=1,  hw_port=54),
        _sfp(label="3",  adj_label=None,  phy_addr=2,  hw_port=55),
        _sfp(label="4",  adj_label=None,  phy_addr=3,  hw_port=56),
        _sfp(label="5",  adj_label=None,  phy_addr=4,  hw_port=57),
        _sfp(label="6",  adj_label=None,  phy_addr=5,  hw_port=58),
        _sfp(label="7",  adj_label=None,  phy_addr=6,  hw_port=59),
        _sfp(label="8",  adj_label=None,  phy_addr=7,  hw_port=60),
        _sfp(label="9",  adj_label=None, phy_addr=8,  hw_port=61),
        _sfp(label="10", adj_label=None, phy_addr=9,  hw_port=62),
        _sfp(label="11", adj_label=None, phy_addr=10, hw_port=63),
        _sfp(label="12", adj_label=None, phy_addr=11, hw_port=64),
        _sfp(label="13", adj_label=None, phy_addr=12, hw_port=33),
        _sfp(label="14", adj_label=None, phy_addr=13, hw_port=43),
        _sfp(label="15", adj_label=None, phy_addr=14, hw_port=44),
        _sfp(label="16", adj_label=None, phy_addr=15, hw_port=34),
        _sfp(label="17", adj_label=None, phy_addr=16, hw_port=45),
        _sfp(label="18", adj_label=None, phy_addr=17, hw_port=46),
        _sfp(label="19", adj_label=None, phy_addr=18, hw_port=35),
        _sfp(label="20", adj_label=None, phy_addr=19, hw_port=38),
        _sfp(label="21", adj_label=None, phy_addr=20, hw_port=39),
        _sfp(label="22", adj_label=None, phy_addr=21, hw_port=36),
        _sfp(label="23", adj_label=None, phy_addr=22, hw_port=40),
        _sfp(label="24", adj_label=None, phy_addr=23, hw_port=41),
        _sfp(label="25", adj_label=None, phy_addr=24, hw_port=1),
        _sfp(label="26", adj_label=None, phy_addr=25, hw_port=6),
        _sfp(label="27", adj_label=None, phy_addr=26, hw_port=7),
        _sfp(label="28", adj_label=None, phy_addr=27, hw_port=2),
        _sfp(label="29", adj_label=None, phy_addr=28, hw_port=8),
        _sfp(label="30", adj_label=None, phy_addr=29, hw_port=9),
        _sfp(label="31", adj_label=None, phy_addr=30, hw_port=3),
        _sfp(label="32", adj_label=None, phy_addr=31, hw_port=11),
        _sfp(label="33", adj_label=None, phy_addr=32, hw_port=12),
        _sfp(label="34", adj_label=None, phy_addr=33, hw_port=4),
        _sfp(label="35", adj_label=None, phy_addr=34, hw_port=13),
        _sfp(label="36", adj_label=None, phy_addr=35, hw_port=14),
        _sfp(label="37", adj_label=None, phy_addr=36, hw_port=27),
        _sfp(label="38", adj_label=None, phy_addr=37, hw_port=28),
        _sfp(label="39", adj_label=None, phy_addr=38, hw_port=29),
        _sfp(label="40", adj_label=None, phy_addr=39, hw_port=30),
        _sfp(label="41", adj_label=None, phy_addr=40, hw_port=31),
        _sfp(label="42", adj_label=None, phy_addr=41, hw_port=32),
        _sfp(label="43", adj_label=None, phy_addr=42, hw_port=21),
        _sfp(label="44", adj_label=None, phy_addr=43, hw_port=22),
        _sfp(label="45", adj_label=None, phy_addr=44, hw_port=23),
        _sfp(label="46", adj_label=None, phy_addr=45, hw_port=24),
        _sfp(label="47", adj_label=None, phy_addr=46, hw_port=25),
        _sfp(label="48", adj_label=None, phy_addr=47, hw_port=26),

        _qsfp(label="49", adj_label=None, phy_addrs=(48,48,48,48), hw_ports=(42,)),
        _qsfp(label="50", adj_label=None, phy_addrs=(49,49,49,49), hw_ports=(47,)),
        _qsfp(label="51", adj_label=None, phy_addrs=(50,50,50,50), hw_ports=(5,)),
        _qsfp(label="52", adj_label=None, phy_addrs=(51,51,51,51), hw_ports=(37,)),
        _qsfp(label="53", adj_label=None, phy_addrs=(52,52,52,52), hw_ports=(15,)),
        _qsfp(label="54", adj_label=None, phy_addrs=(53,53,53,53), hw_ports=(10,)),
        _qsfp(label="55", adj_label=None, phy_addrs=(54,54,54,54), hw_ports=(52,)),
        _qsfp(label="56", adj_label=None, phy_addrs=(55,55,55,55), hw_ports=(20,)),
        _qsfp(label="57", adj_label=None, phy_addrs=(56,56,56,56), hw_ports=(50,)),
        _qsfp(label="58", adj_label=None, phy_addrs=(57,57,57,57), hw_ports=(18,)),
        _qsfp(label="59", adj_label=None, phy_addrs=(58,58,58,58), hw_ports=(48,49)),
        _qsfp(label="60", adj_label=None, phy_addrs=(59,59,59,59), hw_ports=(16,17)),
    )


class MlxSX1400CPLD(cumulus.cpld.SysFS):
    path = None

# Base class for power supply unit in MSX1400
class MlxSX1400PSU(cumulus.sunit.PSU):
    """
        Led is controlled by HW and there is no way to
        read the LED color
    """
    _state_ledcolor_map = {}
    led_location = None

    def _calculate_state(self, platform, expected_unit_count):
        """
        BAD - PSU is not present/working
        OK  - Running Ok
        """
        state = self.default_state
        for power_source in self.sources:
            status =  self.get_cpld_status(platform, power_source)
            if status != '1':
                state = 'BAD'
                break

        return state

class MlxSX1400PSU_1(MlxSX1400PSU):
    sources = ('psu_pwr1_all_ok',)

    def __init__(self):
        cumulus.sunit.PSU.__init__(self, 1)

# System Fan
class MlxSX1400Fan(cumulus.sunit.Fan):
    _state_ledcolor_map = {}
    led_location = None

    def _calculate_state(self, platform, expected_unit_count):
        """
        BAD - One of FAN is FAN_NOT_SPINNING or NOT_INSTALLED
        OK  - Two Fans are running OK
       """
        state = self.default_state
        for fan_source in self.sources:
            status =  self.get_cpld_status(platform, fan_source)
            if status != '1':
                state = 'BAD'
                break

        return state

class MlxSX1400Fan_1(MlxSX1400Fan):
   sources = ('fan1_all_ok',)
   operational_status_file = "fan1"

   def __init__(self):
        cumulus.sunit.Fan.__init__(self, "fan_1")

# Entire System status
class MlxSX1400System(cumulus.sunit.System):
    sources = ('psu_power', 'fan1')
    led_location = "led_status"
    _state_ledcolor_map = { 'ERROR' : 'green_blinking',
                            'OK' : 'green' }

# System LEDs present in switch
class MlxSX1400SysLED():
    sunits = (
            MlxSX1400PSU_1(),
            MlxSX1400Fan_1(),
            MlxSX1400System(),
        )

class MlxSX1400(cumulus.platform.Platform):
    name = 'mlnx,x86_MSX1400'
    chip = cumulus.platform.SwitchXTwoChip()
    switch = MlxSX1400Switch(chip)
    cpld = MlxSX1400CPLD()
    sysled = MlxSX1400SysLED()

Platform = MlxSX1400
