#
# Copyright (c) 2018 Cumulus Networks, Inc.  All Rights Reserved.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import syslog

class QuantaIX8SFP28(cumulus.platform.TridentThree25GPort):

    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.TridentThree25GPort.__init__(self,
                                                  label=label,
                                                  serdes=serdes,
                                                  leds=None,
                                                  phy_types=None,
                                                  phy_addrs=None,
                                                  phy_layer=sfp.type.upper(),
                                                  rx_lane_map=rx_lane_map,
                                                  rx_polarity_flip=rx_polarity_flip,
                                                  tx_lane_map=tx_lane_map,
                                                  tx_polarity_flip=tx_polarity_flip,
                                                  pre_emphasis=pre_emphasis,
                                                  oversub=True,
                                                  unit=0)
        self.sfp = sfp

class QuantaIX8QSFP28(cumulus.platform.TridentThree100GPort):

    def __init__(self, label, serdes, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.TridentThree100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp

class QuantaIX8SwitchA(cumulus.platform.BCMSwitch):
    _sfp28 = QuantaIX8SFP28
    _qsfp28 = QuantaIX8QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        # ******************************************************************
        # This port mapping table applies to the Rev A boards only
        # ******************************************************************
        _sfp28(label="1",   serdes=( 48,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),

        _sfp28(label="2",   serdes=( 56,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="3",   serdes=( 57,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="4",   serdes=( 58,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="5",   serdes=( 59,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),

        _sfp28(label="6",   serdes=( 60,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="7",   serdes=( 61,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="8",   serdes=( 62,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="9",   serdes=( 63,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),

        _sfp28(label="10",  serdes=( 49,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="11",  serdes=( 50,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="12",  serdes=( 51,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),

        _sfp28(label="13",  serdes=(  0,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="14",  serdes=(  1,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="15",  serdes=(  2,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="16",  serdes=(  3,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),

        _sfp28(label="17",  serdes=(  4,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="18",  serdes=(  5,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="19",  serdes=(  6,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="20",  serdes=(  7,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="21",  serdes=( 12,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="22",  serdes=( 13,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="23",  serdes=( 14,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="24",  serdes=( 15,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="25",  serdes=( 20,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="26",  serdes=( 21,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="27",  serdes=( 22,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="28",  serdes=( 23,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="29",  serdes=( 28,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="30",  serdes=( 29,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="31",  serdes=( 30,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="32",  serdes=( 31,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),

        _sfp28(label="33",  serdes=( 32,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="34",  serdes=( 33,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="35",  serdes=( 34,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="36",  serdes=( 35,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="37",  serdes=( 40,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="38",  serdes=( 41,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="39",  serdes=( 42,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="40",  serdes=( 43,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="41",  serdes=( 84,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="42",  serdes=( 85,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="43",  serdes=( 86,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="44",  serdes=( 87,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True, ),

        _sfp28(label="45",  serdes=( 92,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="46",  serdes=( 93,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="47",  serdes=( 94,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="48",  serdes=( 95,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True, ),

        # ******************************************************************
        # This port mapping table applies to the Rev A boards only
        # ******************************************************************
        _qsfp28(label="49", serdes=( 96,  97,  98,  99), tx_lane_map=0x0312, rx_lane_map=0x1023, tx_polarity_flip=0x000c, rx_polarity_flip=0x0000,),
        _qsfp28(label="50", serdes=(104, 105, 106, 107), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x0000, rx_polarity_flip=0x000a,),
        _qsfp28(label="51", serdes=(112, 113, 114, 115), tx_lane_map=0x3120, rx_lane_map=0x1023, tx_polarity_flip=0x000e, rx_polarity_flip=0x0007,),
        _qsfp28(label="52", serdes=(120, 121, 122, 123), tx_lane_map=0x0132, rx_lane_map=0x0321, tx_polarity_flip=0x0003, rx_polarity_flip=0x000a,),
        _qsfp28(label="53", serdes=(124, 125, 126, 127), tx_lane_map=0x2130, rx_lane_map=0x3102, tx_polarity_flip=0x0006, rx_polarity_flip=0x000d,),
        _qsfp28(label="54", serdes=( 64,  65,  66,  67), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x000f, rx_polarity_flip=0x0004,),
        _qsfp28(label="55", serdes=( 68,  69,  70,  71), tx_lane_map=0x2130, rx_lane_map=0x3102, tx_polarity_flip=0x000c, rx_polarity_flip=0x000a,),
        _qsfp28(label="56", serdes=( 76,  77,  78,  79), tx_lane_map=0x0213, rx_lane_map=0x0312, tx_polarity_flip=0x0008, rx_polarity_flip=0x0007,),
    )

class QuantaIX8SwitchB(cumulus.platform.BCMSwitch):
    _sfp28 = QuantaIX8SFP28
    _qsfp28 = QuantaIX8QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        # ******************************************************************
        # This port mapping table applies to the Rev B boards only
        # ******************************************************************
        _sfp28(label="1",   serdes=( 48,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),

        _sfp28(label="2",   serdes=( 56,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="3",   serdes=( 57,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="4",   serdes=( 58,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="5",   serdes=( 59,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),

        _sfp28(label="6",   serdes=( 60,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="7",   serdes=( 61,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="8",   serdes=( 62,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="9",   serdes=( 63,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),

        _sfp28(label="10",  serdes=( 49,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="11",  serdes=( 50,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="12",  serdes=( 51,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),

        _sfp28(label="13",  serdes=(  0,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="14",  serdes=(  1,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="15",  serdes=(  2,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="16",  serdes=(  3,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),

        _sfp28(label="17",  serdes=(  4,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="18",  serdes=(  5,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="19",  serdes=(  6,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="20",  serdes=(  7,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="21",  serdes=( 12,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="22",  serdes=( 13,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="23",  serdes=( 14,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="24",  serdes=( 15,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="25",  serdes=( 20,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="26",  serdes=( 21,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="27",  serdes=( 22,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="28",  serdes=( 23,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="29",  serdes=( 28,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="30",  serdes=( 29,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="31",  serdes=( 30,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="32",  serdes=( 31,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),

        _sfp28(label="33",  serdes=( 32,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="34",  serdes=( 33,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="35",  serdes=( 34,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="36",  serdes=( 35,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="37",  serdes=( 40,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="38",  serdes=( 41,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="39",  serdes=( 42,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="40",  serdes=( 43,), tx_lane_map=0x0123, rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="41",  serdes=( 84,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="42",  serdes=( 85,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="43",  serdes=( 86,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="44",  serdes=( 87,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True, ),

        _sfp28(label="45",  serdes=( 92,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="46",  serdes=( 93,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="47",  serdes=( 94,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="48",  serdes=( 95,), tx_lane_map=0x2301, rx_lane_map=0x2301, tx_polarity_flip=False, rx_polarity_flip=True, ),

        # ******************************************************************
        # This port mapping table applies to the Rev B boards only
        # ******************************************************************
        _qsfp28(label="49", serdes=( 96,  97,  98,  99), tx_lane_map=0x0312, rx_lane_map=0x1023, tx_polarity_flip=0x000c, rx_polarity_flip=0x0000,),
        _qsfp28(label="50", serdes=(104, 105, 106, 107), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x0000, rx_polarity_flip=0x000a,),
        _qsfp28(label="51", serdes=(112, 113, 114, 115), tx_lane_map=0x3120, rx_lane_map=0x1023, tx_polarity_flip=0x000e, rx_polarity_flip=0x0007,),
        _qsfp28(label="52", serdes=(120, 121, 122, 123), tx_lane_map=0x0132, rx_lane_map=0x0321, tx_polarity_flip=0x0003, rx_polarity_flip=0x000a,),
        _qsfp28(label="53", serdes=(124, 125, 126, 127), tx_lane_map=0x2130, rx_lane_map=0x2103, tx_polarity_flip=0x0006, rx_polarity_flip=0x0004,),
        _qsfp28(label="54", serdes=( 64,  65,  66,  67), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x000f, rx_polarity_flip=0x0004,),
        _qsfp28(label="55", serdes=( 68,  69,  70,  71), tx_lane_map=0x3120, rx_lane_map=0x0123, tx_polarity_flip=0x000e, rx_polarity_flip=0x0002,),
        _qsfp28(label="56", serdes=( 76,  77,  78,  79), tx_lane_map=0x0213, rx_lane_map=0x0321, tx_polarity_flip=0x0008, rx_polarity_flip=0x0009,),
    )

class QuantaIX8SwitchC(cumulus.platform.BCMSwitch):
    _sfp28 = QuantaIX8SFP28
    _qsfp28 = QuantaIX8QSFP28
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        # ******************************************************************
        # This port mapping table applies to the Rev C boards only
        # ******************************************************************
        _sfp28(label="1",   serdes=( 59,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="2",   serdes=( 58,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="3",   serdes=( 57,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="4",   serdes=( 56,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),

        _sfp28(label="5",   serdes=( 63,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="6",   serdes=( 62,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="7",   serdes=( 61,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="8",   serdes=( 60,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),

        _sfp28(label="9",   serdes=( 48,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="10",  serdes=( 49,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="11",  serdes=( 50,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="12",  serdes=( 51,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),

        _sfp28(label="13",  serdes=(  3,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="14",  serdes=(  2,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="15",  serdes=(  1,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="16",  serdes=(  0,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),

        _sfp28(label="17",  serdes=(  7,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="18",  serdes=(  6,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="19",  serdes=(  5,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="20",  serdes=(  4,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="21",  serdes=( 15,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="22",  serdes=( 14,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="23",  serdes=( 13,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="24",  serdes=( 12,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="25",  serdes=( 23,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="26",  serdes=( 22,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="27",  serdes=( 21,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="28",  serdes=( 20,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="29",  serdes=( 31,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="30",  serdes=( 30,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="31",  serdes=( 29,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="32",  serdes=( 28,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),

        _sfp28(label="33",  serdes=( 35,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="34",  serdes=( 34,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="35",  serdes=( 33,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="36",  serdes=( 32,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="37",  serdes=( 43,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="38",  serdes=( 42,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="39",  serdes=( 41,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="40",  serdes=( 40,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),

        _sfp28(label="41",  serdes=( 85,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False, ),
        _sfp28(label="42",  serdes=( 84,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=False,),
        _sfp28(label="43",  serdes=( 87,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="44",  serdes=( 86,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),

        _sfp28(label="45",  serdes=( 93,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="46",  serdes=( 92,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),
        _sfp28(label="47",  serdes=( 95,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=True,  rx_polarity_flip=True, ),
        _sfp28(label="48",  serdes=( 94,), tx_lane_map=0x3210, rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True, ),

        # ******************************************************************
        # This port mapping table applies to the Rev C boards only
        # ******************************************************************
        _qsfp28(label="49", serdes=( 96,  97,  98,  99), tx_lane_map=0x0312, rx_lane_map=0x1023, tx_polarity_flip=0x000c, rx_polarity_flip=0x0000,),
        _qsfp28(label="50", serdes=(104, 105, 106, 107), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x0000, rx_polarity_flip=0x000a,),
        _qsfp28(label="51", serdes=(112, 113, 114, 115), tx_lane_map=0x3120, rx_lane_map=0x1023, tx_polarity_flip=0x0001, rx_polarity_flip=0x0007,),
        _qsfp28(label="52", serdes=(120, 121, 122, 123), tx_lane_map=0x0132, rx_lane_map=0x0321, tx_polarity_flip=0x0003, rx_polarity_flip=0x000a,),
        _qsfp28(label="53", serdes=( 76,  77,  78,  79), tx_lane_map=0x2130, rx_lane_map=0x3102, tx_polarity_flip=0x0003, rx_polarity_flip=0x0003,),
        _qsfp28(label="54", serdes=( 64,  65,  66,  67), tx_lane_map=0x0123, rx_lane_map=0x0321, tx_polarity_flip=0x000f, rx_polarity_flip=0x0004,),
        _qsfp28(label="55", serdes=( 68,  69,  70,  71), tx_lane_map=0x3120, rx_lane_map=0x0123, tx_polarity_flip=0x000e, rx_polarity_flip=0x0002,),
        _qsfp28(label="56", serdes=(124, 125, 126, 127), tx_lane_map=0x0213, rx_lane_map=0x0321, tx_polarity_flip=0x0008, rx_polarity_flip=0x0003,),
    )

vhwmon_path = '/sys/class/misc/vhwmon'

# Intel Rangeley CPU
class QuantaIX8CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    no_shutdown = True

class QuantaIX8CPUTemp1(QuantaIX8CoreTemp):
    name = 'Temp1'
    description = 'Core 0'
    driver_hwmon = ['temp2']

class QuantaIX8CPUTemp2(QuantaIX8CoreTemp):
    name = 'Temp2'
    description = 'Core 1'
    driver_hwmon = ['temp3']

class QuantaIX8CPUTemp3(QuantaIX8CoreTemp):
    name = 'Temp3'
    description = 'Core 2'
    driver_hwmon = ['temp4']

class QuantaIX8CPUTemp4(QuantaIX8CoreTemp):
    name = 'Temp4'
    description = 'Core 3'
    driver_hwmon = ['temp5']

# Trident3 Sensors
# *** TBD ***

# BMC Sensors
class QuantaIX8BMCTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + '/bmc:temp'

class QuantaIX8BMCTemp1(QuantaIX8BMCTemp):
    name = 'Temp14'
    description = 'Temp_CPU'
    driver_hwmon = ['temp1']

class QuantaIX8BMCTemp2(QuantaIX8BMCTemp):
    name = 'Temp15'
    description = 'Temp_Ambient_0'
    driver_hwmon = ['temp2']

class QuantaIX8BMCTemp3(QuantaIX8BMCTemp):
    name = 'Temp16'
    description = 'Temp_Ambient_1'
    driver_hwmon = ['temp3']

class QuantaIX8BMCTemp4(QuantaIX8BMCTemp):
    name = 'Temp17'
    description = 'Temp_Ambient_2'
    driver_hwmon = ['temp4']

class QuantaIX8BMCTemp5(QuantaIX8BMCTemp):
    name = 'Temp18'
    description = 'Temp_Ambient_3'
    driver_hwmon = ['temp5']

class QuantaIX8BMCTemp6(QuantaIX8BMCTemp):
    name = 'Temp19'
    description = 'Temp_Ambient_4'
    driver_hwmon = ['temp6']

class QuantaIX8BMCTemp7(QuantaIX8BMCTemp):
    name = 'Temp20'
    description = 'Temp_Ambient_5'
    driver_hwmon = ['temp7']

class QuantaIX8BMCTemp8(QuantaIX8BMCTemp):
    name = 'Temp21'
    description = 'Temp_Ambient_6'
    driver_hwmon = ['temp8']

# Fans
class QuantaIX8Fan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + '/bmc:fan'
    no_pwm = True

class QuantaIX8Fan1(QuantaIX8Fan):
    name = 'Fan1'
    description = 'Fan 1_1'
    driver_hwmon = ['fan1']

class QuantaIX8Fan2(QuantaIX8Fan):
    name = 'Fan2'
    description = 'Fan 1_2'
    driver_hwmon = ['fan2']

class QuantaIX8Fan3(QuantaIX8Fan):
    name = 'Fan3'
    description = 'Fan 2_1'
    driver_hwmon = ['fan3']

class QuantaIX8Fan4(QuantaIX8Fan):
    name = 'Fan4'
    description = 'Fan 2_2'
    driver_hwmon = ['fan4']

class QuantaIX8Fan5(QuantaIX8Fan):
    name = 'Fan5'
    description = 'Fan 3_1'
    driver_hwmon = ['fan5']

class QuantaIX8Fan6(QuantaIX8Fan):
    name = 'Fan6'
    description = 'Fan 3_2'
    driver_hwmon = ['fan6']

class QuantaIX8Fan7(QuantaIX8Fan):
    name = 'Fan7'
    description = 'Fan 4_1'
    driver_hwmon = ['fan7']

class QuantaIX8Fan8(QuantaIX8Fan):
    name = 'Fan8'
    description = 'Fan 4_2'
    driver_hwmon = ['fan8']

class QuantaIX8Fan9(QuantaIX8Fan):
    name = 'Fan9'
    description = 'Fan 5_1'
    driver_hwmon = ['fan9']

class QuantaIX8Fan10(QuantaIX8Fan):
    name = 'Fan10'
    description = 'Fan 5_2'
    driver_hwmon = ['fan10']

class QuantaIX8Fan11(QuantaIX8Fan):
    name = 'Fan11'
    description = 'Fan 6_1'
    driver_hwmon = ['fan11']

class QuantaIX8Fan12(QuantaIX8Fan):
    name = 'Fan12'
    description = 'Fan 6_2'
    driver_hwmon = ['fan12']

# PSUs
class QuantaIX8PSU(cumulus.sensors.PSU_Unit):
    driver_hwmon = ['power1']

class QuantaIX8PSU1(QuantaIX8PSU):
    name = 'PSU1'
    driver_path = vhwmon_path + '/bmc:psu1'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr1_all_ok'] = val
        QuantaIX8PSU1.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class QuantaIX8PSU2(QuantaIX8PSU):
    name = 'PSU2'
    driver_path = vhwmon_path + '/bmc:psu2'
    psu_ok = 0

    def get_status(self):
        val = self.attrs.get('power1_input', 0)
        if val != 0:
            val = 1
        self.attrs['psu_pwr2_all_ok'] = val
        QuantaIX8PSU2.psu_ok = val
        return cumulus.sensors.PSU_Unit.get_status(self)

class QuantaIX8PSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    no_pwm = True

    def get_status(self):
        if not self.psu.psu_ok:
            return self.update_status(None, 'not_installed')
        return cumulus.sensors.Fan_Unit.get_status(self)

class QuantaIX8PSU1Fan1(QuantaIX8PSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan1'
    psu = QuantaIX8PSU1()
    driver_path = psu.driver_path

class QuantaIX8PSU2Fan1(QuantaIX8PSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan1'
    psu = QuantaIX8PSU2()
    driver_path = psu.driver_path

class QuantaIX8Sensors():
    units = (QuantaIX8CPUTemp1(),
             QuantaIX8CPUTemp2(),
             QuantaIX8CPUTemp3(),
             QuantaIX8CPUTemp4(),
             QuantaIX8BMCTemp1(),
             QuantaIX8BMCTemp2(),
             QuantaIX8BMCTemp3(),
             QuantaIX8BMCTemp4(),
             QuantaIX8BMCTemp5(),
             QuantaIX8BMCTemp6(),
             QuantaIX8BMCTemp7(),
             QuantaIX8BMCTemp8(),
             QuantaIX8Fan1(),
             QuantaIX8Fan2(),
             QuantaIX8Fan3(),
             QuantaIX8Fan4(),
             QuantaIX8Fan5(),
             QuantaIX8Fan6(),
             QuantaIX8Fan7(),
             QuantaIX8Fan8(),
             QuantaIX8Fan9(),
             QuantaIX8Fan10(),
             QuantaIX8Fan11(),
             QuantaIX8Fan12(),
             QuantaIX8PSU1(),
             QuantaIX8PSU2(),
             QuantaIX8PSU1Fan1(),
             QuantaIX8PSU2Fan1(),
             )

class QuantaIX8BMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'

def board_rev():
    try:
        plat = subprocess.check_output(('platform-detect', '-m'))
    except OSError:
        return None
    if plat.strip() != 'quanta,ix8_rglbmc':
        return None

    try:
        b_info = subprocess.check_output('/usr/cumulus/bin/decode-syseeprom')
    except:
        b_info = ''
    # Part Number Format
    # TLV Name             Code Len Value
    # -------------------- ---- --- -----
    # Label Revision       0x27   3 C2A
    rev = ''
    for l in b_info.split('\n'):
        if 'Label Revision' in l:
            rev = l.split()[4]
            break
    if rev.startswith('C'):
        return 'C'
    elif rev.startswith('B'):
        syslog.syslog(syslog.LOG_WARNING, "Rev B board detected.")
        return 'B'
    elif rev.startswith('A'):
        syslog.syslog(syslog.LOG_WARNING, "Rev A board detected.")
        return 'A'
    else:
        syslog.syslog(syslog.LOG_WARNING, "Could not determine board Rev.  Assuming production Rev.")
        return None

class QuantaIX8(cumulus.platform.Platform):
    name = 'quanta,ix8_rglbmc'
    chip = cumulus.platform.TridentThreeX7_56873_Chip()

    # Port mappings are different in each rev.  If we can't determine the rev,
    # then assume "C" which is the production rev.
    rev = board_rev()
    if rev == 'A':
        switch = QuantaIX8SwitchA(chip)
    elif rev == 'B':
        switch = QuantaIX8SwitchB(chip)
    else:
        switch = QuantaIX8SwitchC(chip)

    bmc = QuantaIX8BMC()
    sensors = QuantaIX8Sensors()
    statusleds = None
    pwms = None

Platform = QuantaIX8
