#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaLB8SFPp(cumulus.platform.Trident10GPort):
    def __init__(self, label, serdes, phy_addr):
        cumulus.platform.Trident10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_type="8754",
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=None,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSWalk(label, 'sfp+')

class QuantaLB8PSUCombo(cumulus.sunit.PSUCombo):
    pass

class QuantaLB8Fan(cumulus.sunit.Fan):
    pass

class QuantaLB8System(cumulus.sunit.System):
    pass

class QuantaLB8Switch(cumulus.platform.BCMSwitch):
    _sfp = QuantaLB8SFPp
    has_phy_ucode_spi_rom = True
    has_phy_ucode_mdio = True
    ports = (
        _sfp(label="1",  serdes=8,  phy_addr=0x4),
        _sfp(label="2",  serdes=9,  phy_addr=0x5),
        _sfp(label="3",  serdes=10, phy_addr=0x6),
        _sfp(label="4",  serdes=11, phy_addr=0x7),
        _sfp(label="5",  serdes=12, phy_addr=0x8),
        _sfp(label="6",  serdes=13, phy_addr=0x9),
        _sfp(label="7",  serdes=14, phy_addr=0xa),
        _sfp(label="8",  serdes=15, phy_addr=0xb),
        _sfp(label="9",  serdes=16, phy_addr=0xc),
        _sfp(label="10", serdes=17, phy_addr=0xd),
        _sfp(label="11", serdes=18, phy_addr=0xe),
        _sfp(label="12", serdes=19, phy_addr=0xf),
        _sfp(label="13", serdes=24, phy_addr=0x10),
        _sfp(label="14", serdes=25, phy_addr=0x11),
        _sfp(label="15", serdes=26, phy_addr=0x12),
        _sfp(label="16", serdes=27, phy_addr=0x13),
        _sfp(label="17", serdes=28, phy_addr=0x24),
        _sfp(label="18", serdes=29, phy_addr=0x25),
        _sfp(label="19", serdes=30, phy_addr=0x26),
        _sfp(label="20", serdes=31, phy_addr=0x27),
        _sfp(label="21", serdes=32, phy_addr=0x28),
        _sfp(label="22", serdes=33, phy_addr=0x29),
        _sfp(label="23", serdes=34, phy_addr=0x2a),
        _sfp(label="24", serdes=35, phy_addr=0x2b),
        _sfp(label="25", serdes=44, phy_addr=0x2c),
        _sfp(label="26", serdes=45, phy_addr=0x2d),
        _sfp(label="27", serdes=46, phy_addr=0x2e),
        _sfp(label="28", serdes=47, phy_addr=0x2f),
        _sfp(label="29", serdes=48, phy_addr=0x30),
        _sfp(label="30", serdes=49, phy_addr=0x31),
        _sfp(label="31", serdes=50, phy_addr=0x32),
        _sfp(label="32", serdes=51, phy_addr=0x33),
        _sfp(label="33", serdes=52, phy_addr=0x44),
        _sfp(label="34", serdes=53, phy_addr=0x45),
        _sfp(label="35", serdes=54, phy_addr=0x46),
        _sfp(label="36", serdes=55, phy_addr=0x47),
        _sfp(label="37", serdes=60, phy_addr=0x48),
        _sfp(label="38", serdes=61, phy_addr=0x49),
        _sfp(label="39", serdes=62, phy_addr=0x4a),
        _sfp(label="40", serdes=63, phy_addr=0x4b),
        _sfp(label="41", serdes=64, phy_addr=0x4c),
        _sfp(label="42", serdes=65, phy_addr=0x4d),
        _sfp(label="43", serdes=66, phy_addr=0x4e),
        _sfp(label="44", serdes=67, phy_addr=0x4f),
        _sfp(label="45", serdes=68, phy_addr=0x50),
        _sfp(label="46", serdes=69, phy_addr=0x51),
        _sfp(label="47", serdes=70, phy_addr=0x52),
        _sfp(label="48", serdes=71, phy_addr=0x53),
    )

class QuantaLB8(cumulus.platform.Platform):
    name = 'quanta,lb8'
    chip = cumulus.platform.TridentChip()
    switch = QuantaLB8Switch(chip)

Platform = QuantaLB8
