#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class QuantaLY8_Fan(cumulus.sensors.Fan_Unit):
    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-004e'
                driver_path = os.path.join(pci_path, x, c)
                break
            
    fan_min = 6500
    fan_max = 18000
    rpmmode = 1

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '0':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class QuantaLY8CoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'

    temp_fan_min = 65
    temp_max = 90
    temp_crit = 98

class QuantaLY8CY8Temp(cumulus.sensors.Temp_Unit):
    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-004e'
                driver_path = os.path.join(pci_path, x, c)
                break

    temp_fan_min = 45
    temp_max = 55
    # OTP temp among sensors specified by Quanta is between 60C-70C
    # Critical temp specified is between 53C - 58C
    temp_crit = 65

class QuantaLY8PSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0
        if ((self.get_gpio_value(self.psu_ac_ok) == '0') and
            (self.get_gpio_value(self.psu_dc_ok) == '1')):
            self.attrs['psu_pwr1_all_ok'] = 1
        else:
            self.attrs['psu_pwr1_all_ok'] = 0
        return 0

class QuantaLY8LED(cumulus.sensors.StatusLED_Unit):
    good_led_color = '1'  # Green
    bad_led_color = '0'   # Off

    def led_update(self, color):
        #print 'loc: %s color %s' %(self.led_location, color)
        self.set_gpio_value(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '0':
            color = 'off'
        else:
            color = 'green'
        return self.name, color

class QuantaLY8PSU1LED(QuantaLY8LED):
    name = 'PSU1'
    led_location = 'psu1_green_led'
    sources = ['PSU1']

class QuantaLY8PSU2LED(QuantaLY8LED):
    name = 'PSU2'
    led_location = 'psu2_green_led'
    sources = ['PSU2']

class QuantaLY8FanLED(QuantaLY8LED):
    name = 'Fans'
    led_location = 'fan_green_led'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6']

class QuantaLY8PSU1(QuantaLY8PSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_ac_ok = 'psu_pwr1_ac_ok'
    psu_dc_ok = 'psu_pwr1_dc_ok'

class QuantaLY8PSU2(QuantaLY8PSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_ac_ok = 'psu_pwr2_ac_ok'
    psu_dc_ok = 'psu_pwr2_dc_ok'

class QuantaLY8Fan1(QuantaLY8_Fan):
    fan_present = 'fan1_present'
    name = 'Fan1'
    driver_hwmon = ['fan1']

class QuantaLY8Fan2(QuantaLY8_Fan):
    fan_present = 'fan1_present'
    name = 'Fan2'
    driver_hwmon = ['fan5']

class QuantaLY8Fan3(QuantaLY8_Fan):
    fan_present = 'fan2_present'
    name = 'Fan3'
    driver_hwmon = ['fan2']

class QuantaLY8Fan4(QuantaLY8_Fan):
    fan_present = 'fan2_present'
    name = 'Fan4'
    driver_hwmon = ['fan6']

class QuantaLY8Fan5(QuantaLY8_Fan):
    fan_present = 'fan3_present'
    name = 'Fan5'
    driver_hwmon = ['fan3']

class QuantaLY8Fan6(QuantaLY8_Fan):
    fan_present = 'fan3_present'
    name = 'Fan6'
    driver_hwmon = ['fan7']

class QuantaLY8Temp1(QuantaLY8CoreTemp):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class QuantaLY8Temp2(QuantaLY8CoreTemp):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class QuantaLY8Temp3(QuantaLY8CoreTemp):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class QuantaLY8Temp4(QuantaLY8CoreTemp):
    name = 'Temp4'
    driver_hwmon = ['temp5']

class QuantaLY8Temp5(QuantaLY8CoreTemp):
    name = 'Temp5'
    driver_hwmon = ['temp6']

class QuantaLY8Temp6(QuantaLY8CoreTemp):
    name = 'Temp6'
    driver_hwmon = ['temp7']

class QuantaLY8Temp7(QuantaLY8CoreTemp):
    name = 'Temp7'
    driver_hwmon = ['temp8']

class QuantaLY8Temp8(QuantaLY8CoreTemp):
    name = 'Temp8'
    driver_hwmon = ['temp9']

class QuantaLY8Temp9(QuantaLY8CY8Temp):
    name = 'Temp9'
    driver_hwmon = ['temp1']
    description = 'Rear Outlet Air Temp sensor'

class QuantaLY8Temp10(QuantaLY8CY8Temp):
    name = 'Temp10'
    driver_hwmon = ['temp2']
    description = 'Front Outlet Air Temp sensor'

class QuantaLY8Temp11(QuantaLY8CY8Temp):
    name = 'Temp11'
    driver_hwmon = ['temp3']
    description = 'Front Outlet Air Temp sensor'

class QuantaLY8Temp12(QuantaLY8CY8Temp):
    name = 'Temp12'
    driver_hwmon = ['temp5']
    description = 'CPU Board Temp sensor'

class QuantaLY8Temp13(QuantaLY8CY8Temp):
    name = 'Temp13'
    driver_hwmon = ['temp6']
    description = 'CPU Board Temp sensor'

class QuantaLY8Temp14(cumulus.sensors.Temp_Unit):
    name = 'Temp14'
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_hwmon = ['temp1']
    temp_fan_min = 65
    temp_max = 90
    temp_crit = 95

    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-001a'
                driver_path = os.path.join(pci_path, x, c)
                break

class QuantaLY8Sensors():
    units = (QuantaLY8Temp1(),
             QuantaLY8Temp2(),
             QuantaLY8Temp3(),
             QuantaLY8Temp4(),
             QuantaLY8Temp5(),
             QuantaLY8Temp6(),
             QuantaLY8Temp7(),
             QuantaLY8Temp8(),
             QuantaLY8Temp9(),
             QuantaLY8Temp10(),
             QuantaLY8Temp11(),
             QuantaLY8Temp12(),
             QuantaLY8Temp13(),
             QuantaLY8Temp14(),
             QuantaLY8Fan1(),
             QuantaLY8Fan2(),
             QuantaLY8Fan3(),
             QuantaLY8Fan4(),
             QuantaLY8Fan5(),
             QuantaLY8Fan6(),
             QuantaLY8PSU1(),
             QuantaLY8PSU2(),
             )

class QuantaLY8PWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (QuantaLY8Fan1(),
            QuantaLY8Fan2(),
            QuantaLY8Fan3(),
            QuantaLY8Fan4(),
            QuantaLY8Fan5(),
            QuantaLY8Fan6(),
            )
    temps = (QuantaLY8Temp1(),
             QuantaLY8Temp2(),
             QuantaLY8Temp3(),
             QuantaLY8Temp4(),
             QuantaLY8Temp5(),
             QuantaLY8Temp6(),
             QuantaLY8Temp7(),
             QuantaLY8Temp8(),
             QuantaLY8Temp9(),
             QuantaLY8Temp10(),
             QuantaLY8Temp11(),
             QuantaLY8Temp12(),
             QuantaLY8Temp13(),
             QuantaLY8Temp14(),
             )

class QuantaLY8PWMs():
    units = (QuantaLY8PWM1(),
             )

class QuantaLY8LEDs():
    units = (QuantaLY8PSU1LED(),
             QuantaLY8PSU2LED(),
             QuantaLY8FanLED(),
             )

class QuantaLY8SFPp(cumulus.platform.TridentTwo10GPort):
    _set_led = '''
;;; QuantaLY8SFPp
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed should always be zero;
;;;
;;; 2nd bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
set_any_led:
set_led:
        port 1
        pushst PORTSTATUS_ZERO
        pack

        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''
    def __init__(self, label, serdes, leds):
        cumulus.platform.TridentTwo10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=0)

        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls


class QuantaLY8QSFPp(cumulus.platform.TridentTwo40GPort):
    _set_led = '''
;;; QuantaLY8QSFPp
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed is always zero
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
set_led:
        call    set_any_led    ; set the first led green/off

        push    PORTSTATUS_ZERO ; 0, 0 => led 2 off
        pack
        push    PORTSTATUS_ZERO
        pack
        push    PORTSTATUS_ZERO ; 0, 0 => led 3 off
        pack
        push    PORTSTATUS_ZERO
        pack
        push    PORTSTATUS_ZERO ; 0, 0 => led 4 off
        pack
        push    PORTSTATUS_ZERO
        pack
        ret

'''

    _set_led_4x10 = '''
;;; QuantaLY8QSFPp_4x10
;;
;;  For 4x10G mode, we use an LED for each port.
;;
set_led:
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led

        ret

'''
    def __init__(self, label, serdes, leds, phy_addrs, prim_offset,
                 phy_id1, sys_interface, phy_types):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_addrs=phy_addrs,
                                                 phy_layer=sfp.type.upper(),
                                                 phy_types=phy_types,
                                                 prim_offset=prim_offset,
                                                 phy_id1=phy_id1,
                                                 sys_interface=sys_interface,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = sfp
        if (self.label == "53") or (self.label == "54"):
            self._serdes_intf_map = {40:'SR4'}

    @property
    def max_logical_ports(self):
        if (self.label == "53" or self.label == "54"):
            return 1
        else:
            return 4

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None


        return set_calls

class QuantaLY8LED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 128

class QuantaLY8Switch(cumulus.platform.BCMSwitch):
    ledup = QuantaLY8LED
    _sfp = QuantaLY8SFPp
    _qsfp = QuantaLY8QSFPp
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (
        _sfp(label="1",  serdes=(24,), leds=(0, 4)),
        _sfp(label="2",  serdes=(25,), leds=(0, 6)),
        _sfp(label="3",  serdes=(26,), leds=(0, 8)),
        _sfp(label="4",  serdes=(27,), leds=(0, 10)),
        _sfp(label="5",  serdes=(28,), leds=(0, 12)),
        _sfp(label="6",  serdes=(29,), leds=(0, 14)),
        _sfp(label="7",  serdes=(30,), leds=(0, 16)),
        _sfp(label="8",  serdes=(31,), leds=(0, 18)),
        _sfp(label="9",  serdes=(44,), leds=(0, 44)),
        _sfp(label="10", serdes=(45,), leds=(0, 46)),
        _sfp(label="11", serdes=(46,), leds=(0, 48)),
        _sfp(label="12", serdes=(47,), leds=(0, 50)),
        _sfp(label="13", serdes=(48,), leds=(0, 52)),
        _sfp(label="14", serdes=(49,), leds=(0, 54)),
        _sfp(label="15", serdes=(50,), leds=(0, 56)),
        _sfp(label="16", serdes=(51,), leds=(0, 58)),
        _sfp(label="17", serdes=(52,), leds=(0, 60)),
        _sfp(label="18", serdes=(53,), leds=(0, 62)),
        _sfp(label="19", serdes=(54,), leds=(0, 64)),
        _sfp(label="20", serdes=(55,), leds=(0, 66)),
        _sfp(label="21", serdes=(56,), leds=(0, 68)),
        _sfp(label="22", serdes=(57,), leds=(0, 70)),
        _sfp(label="23", serdes=(58,), leds=(0, 72)),
        _sfp(label="24", serdes=(59,), leds=(0, 74)),

        _sfp(label="25", serdes=(76,), leds=(1, 108)),
        _sfp(label="26", serdes=(77,), leds=(1, 110)),
        _sfp(label="27", serdes=(78,), leds=(1, 112)),
        _sfp(label="28", serdes=(79,), leds=(1, 114)),
        _sfp(label="29", serdes=(80,), leds=(1, 116)),
        _sfp(label="30", serdes=(81,), leds=(1, 118)),
        _sfp(label="31", serdes=(82,), leds=(1, 120)),
        _sfp(label="32", serdes=(83,), leds=(1, 122)),

        _sfp(label="33", serdes=(96,), leds=(1, 20)),
        _sfp(label="34", serdes=(97,), leds=(1, 22)),
        _sfp(label="35", serdes=(98,), leds=(1, 24)),
        _sfp(label="36", serdes=(99,), leds=(1, 26)),
        _sfp(label="37", serdes=(100,), leds=(1,28)),
        _sfp(label="38", serdes=(101,), leds=(1,30)),
        _sfp(label="39", serdes=(102,), leds=(1,32)),
        _sfp(label="40", serdes=(103,), leds=(1,34)),
        _sfp(label="41", serdes=(104,), leds=(1,36)),
        _sfp(label="42", serdes=(105,), leds=(1,38)),
        _sfp(label="43", serdes=(106,), leds=(1,40)),
        _sfp(label="44", serdes=(107,), leds=(1,42)),
        _sfp(label="45", serdes=(108,), leds=(1,44)),
        _sfp(label="46", serdes=(109,), leds=(1,46)),
        _sfp(label="47", serdes=(110,), leds=(1,48)),
        _sfp(label="48", serdes=(111,), leds=(1,50)),

        _qsfp(label="49", serdes=(60, 61, 62, 63), leds=(0, 76),
              phy_addrs=None, prim_offset=None, phy_id1=None, sys_interface=None, phy_types=None),
        _qsfp(label="50", serdes=(64, 65, 66, 67), leds=(1, 84),
              phy_addrs=None, prim_offset=None, phy_id1=None, sys_interface=None, phy_types=None),
        _qsfp(label="51", serdes=(68, 69, 70, 71), leds=(1, 92),
              phy_addrs=None, prim_offset=None, phy_id1=None, sys_interface=None, phy_types=None),
        _qsfp(label="52", serdes=(72, 73, 74, 75), leds=(1, 100),
              phy_addrs=None, prim_offset=None, phy_id1=None, sys_interface=None, phy_types=None),
        _qsfp(label="53", serdes=(20, 21, 22, 23), leds=(0, 124),
              phy_addrs = (0x10, 0x11, 0x12, 0x13), prim_offset=0x3500, phy_id1=0x8500, sys_interface=9,
              phy_types = ("84328",) * 4),
        _qsfp(label="54", serdes=(12, 13, 14, 15), leds=(0, 108),
              phy_addrs = (0x14, 0x15, 0x16, 0x17), prim_offset=0x3600, phy_id1=0x8500, sys_interface=9,
              phy_types = ("84328",) * 4),
    )

class QuantaLY8(cumulus.platform.Platform):
    name = 'quanta,ly8_rangeley'
    chip = cumulus.platform.TridentTwo_56854_Chip()
    switch = QuantaLY8Switch(chip)
    pwms = QuantaLY8PWMs()
    sensors = QuantaLY8Sensors()
    statusleds = QuantaLY8LEDs()

Platform = QuantaLY8
