#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

# TODO: Sensors data needs to be updated. Awaiting data from Quanta.
class QuantaLY9Rangeley_Fan(cumulus.sensors.Fan_Unit):
    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-004e'
                driver_path = os.path.join(pci_path, x, c)
                break

    fan_min = 11000
    fan_max = 24000
    rpmmode = 1

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '0':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class QuantaLY9RangeleyCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'

    temp_fan_min = 65
    temp_max = 90
    temp_crit = 98

class QuantaLY9RangeleyCY8Temp(cumulus.sensors.Temp_Unit):
    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-004e'
                driver_path = os.path.join(pci_path, x, c)
                break

    # This box runs hot with PHYs on it
    temp_fan_min = 50
    temp_max = 68
    temp_crit = 73

class QuantaLY9RangeleyPSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0
        if ((self.get_gpio_value(self.psu_ac_ok) == '0') and
            (self.get_gpio_value(self.psu_dc_ok) == '1')):
            self.attrs['psu_pwr1_all_ok'] = 1
        else:
            self.attrs['psu_pwr1_all_ok'] = 0
        return 0

class QuantaLY9RangeleyLED(cumulus.sensors.StatusLED_Unit):
    good_led_color = '1'  # Green
    bad_led_color = '0'   # Off

    def led_update(self, color):
        #print 'loc: %s color %s' %(self.led_location, color)
        self.set_gpio_value(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '0':
            color = 'off'
        else:
            color = 'green'
        return self.name, color

class QuantaLY9RangeleyPSU1LED(QuantaLY9RangeleyLED):
    name = 'PSU1'
    led_location = 'psu1_green_led'
    sources = ['PSU1']

class QuantaLY9RangeleyPSU2LED(QuantaLY9RangeleyLED):
    name = 'PSU2'
    led_location = 'psu2_green_led'
    sources = ['PSU2']

class QuantaLY9RangeleyFanLED(QuantaLY9RangeleyLED):
    name = 'Fans'
    led_location = 'fan_green_led'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6']

class QuantaLY9RangeleyPSU1(QuantaLY9RangeleyPSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_ac_ok = 'psu_pwr1_ac_ok'
    psu_dc_ok = 'psu_pwr1_dc_ok'

class QuantaLY9RangeleyPSU2(QuantaLY9RangeleyPSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_ac_ok = 'psu_pwr2_ac_ok'
    psu_dc_ok = 'psu_pwr2_dc_ok'

class QuantaLY9RangeleyFan1(QuantaLY9Rangeley_Fan):
    fan_present = 'fan1_present'
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_hwmon = ['fan1']

class QuantaLY9RangeleyFan2(QuantaLY9Rangeley_Fan):
    fan_present = 'fan1_present'
    name = 'Fan2'
    description = 'Fan Tray 1'
    driver_hwmon = ['fan5']

class QuantaLY9RangeleyFan3(QuantaLY9Rangeley_Fan):
    fan_present = 'fan2_present'
    name = 'Fan3'
    description = 'Fan Tray 2'
    driver_hwmon = ['fan2']

class QuantaLY9RangeleyFan4(QuantaLY9Rangeley_Fan):
    fan_present = 'fan2_present'
    name = 'Fan4'
    description = 'Fan Tray 2'
    driver_hwmon = ['fan6']

class QuantaLY9RangeleyFan5(QuantaLY9Rangeley_Fan):
    fan_present = 'fan3_present'
    name = 'Fan5'
    description = 'Fan Tray 3'
    driver_hwmon = ['fan3']

class QuantaLY9RangeleyFan6(QuantaLY9Rangeley_Fan):
    fan_present = 'fan3_present'
    name = 'Fan6'
    description = 'Fan Tray 3'
    driver_hwmon = ['fan7']

class QuantaLY9RangeleyTemp1(QuantaLY9RangeleyCoreTemp):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class QuantaLY9RangeleyTemp2(QuantaLY9RangeleyCoreTemp):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class QuantaLY9RangeleyTemp3(QuantaLY9RangeleyCoreTemp):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class QuantaLY9RangeleyTemp4(QuantaLY9RangeleyCoreTemp):
    name = 'Temp4'
    driver_hwmon = ['temp5']

class QuantaLY9RangeleyTemp5(QuantaLY9RangeleyCoreTemp):
    name = 'Temp5'
    driver_hwmon = ['temp6']

class QuantaLY9RangeleyTemp6(QuantaLY9RangeleyCoreTemp):
    name = 'Temp6'
    driver_hwmon = ['temp7']

class QuantaLY9RangeleyTemp7(QuantaLY9RangeleyCoreTemp):
    name = 'Temp7'
    driver_hwmon = ['temp8']

class QuantaLY9RangeleyTemp8(QuantaLY9RangeleyCoreTemp):
    name = 'Temp8'
    driver_hwmon = ['temp9']

class QuantaLY9RangeleyTemp9(QuantaLY9RangeleyCY8Temp):
    name = 'Temp9'
    driver_hwmon = ['temp1']
    description = 'Right Front Outlet Temp sensor'

class QuantaLY9RangeleyTemp10(QuantaLY9RangeleyCY8Temp):
    name = 'Temp10'
    driver_hwmon = ['temp2']
    description = 'Middle Front Outlet Temp sensor'
    # Located near the PHYs
    temp_fan_min = 72
    temp_max = 82
    temp_crit = 90

class QuantaLY9RangeleyTemp11(QuantaLY9RangeleyCY8Temp):
    name = 'Temp11'
    driver_hwmon = ['temp3']
    description = 'Left Front Outlet Temp sensor'
    # Located near the PHYs
    temp_fan_min = 72
    temp_max = 82
    temp_crit = 90

class QuantaLY9RangeleyTemp12(QuantaLY9RangeleyCY8Temp):
    name = 'Temp12'
    driver_hwmon = ['temp4']
    description = "Rear Outlet Temp Sensor"

class QuantaLY9RangeleyTemp13(QuantaLY9RangeleyCY8Temp):
    name = 'Temp13'
    driver_hwmon = ['temp5']
    description = 'CPU Board Temp sensor'

class QuantaLY9RangeleyTemp14(QuantaLY9RangeleyCY8Temp):
    name = 'Temp14'
    driver_hwmon = ['temp6']
    description = 'CPU Board Temp sensor'

class QuantaLY9RangeleyTemp15(cumulus.sensors.Temp_Unit):
    name = 'Temp15'
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_hwmon = ['temp1']
    temp_fan_min = 65
    temp_max = 90
    temp_crit = 95

    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-001a'
                driver_path = os.path.join(pci_path, x, c)
                break

class QuantaLY9RangeleySensors():
    units = (QuantaLY9RangeleyTemp1(),
             QuantaLY9RangeleyTemp2(),
             QuantaLY9RangeleyTemp3(),
             QuantaLY9RangeleyTemp4(),
             QuantaLY9RangeleyTemp5(),
             QuantaLY9RangeleyTemp6(),
             QuantaLY9RangeleyTemp7(),
             QuantaLY9RangeleyTemp8(),
             QuantaLY9RangeleyTemp9(),
             QuantaLY9RangeleyTemp10(),
             QuantaLY9RangeleyTemp11(),
             QuantaLY9RangeleyTemp12(),
             QuantaLY9RangeleyTemp13(),
             QuantaLY9RangeleyTemp14(),
             QuantaLY9RangeleyTemp15(),
             QuantaLY9RangeleyFan1(),
             QuantaLY9RangeleyFan2(),
             QuantaLY9RangeleyFan3(),
             QuantaLY9RangeleyFan4(),
             QuantaLY9RangeleyFan5(),
             QuantaLY9RangeleyFan6(),
             QuantaLY9RangeleyPSU1(),
             QuantaLY9RangeleyPSU2(),
             )

class QuantaLY9RangeleyPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (QuantaLY9RangeleyFan1(),
            QuantaLY9RangeleyFan2(),
            QuantaLY9RangeleyFan3(),
            QuantaLY9RangeleyFan4(),
            QuantaLY9RangeleyFan5(),
            QuantaLY9RangeleyFan6(),
            )
    temps = (QuantaLY9RangeleyTemp1(),
             QuantaLY9RangeleyTemp2(),
             QuantaLY9RangeleyTemp3(),
             QuantaLY9RangeleyTemp4(),
             QuantaLY9RangeleyTemp5(),
             QuantaLY9RangeleyTemp6(),
             QuantaLY9RangeleyTemp7(),
             QuantaLY9RangeleyTemp8(),
             QuantaLY9RangeleyTemp9(),
             QuantaLY9RangeleyTemp10(),
             QuantaLY9RangeleyTemp11(),
             QuantaLY9RangeleyTemp12(),
             QuantaLY9RangeleyTemp13(),
             QuantaLY9RangeleyTemp14(),
             QuantaLY9RangeleyTemp15(),
             )

class QuantaLY9RangeleyPWMs():
    units = (QuantaLY9RangeleyPWM1(),
             )

class QuantaLY9RangeleyLEDs():
    units = (QuantaLY9RangeleyPSU1LED(),
             QuantaLY9RangeleyPSU2LED(),
             QuantaLY9RangeleyFanLED(),
             )

class QuantaLY9Rangeley10GT(cumulus.platform.TridentTwo10GPort):
    _set_led = '''
;;; QuantaLY9Rangeley10GT
;;;
;;; 2 bits per port. Since leds are set by PHY, pack anything.
;;;
set_led:
        port 1
        pushst PORTSTATUS_ZERO
        pack
        port 1
        pushst PORTSTATUS_ZERO
        pack

        ret
'''
    def __init__(self, label, serdes, phy_addrs, leds, mdi_pair_map):
        cumulus.platform.TridentTwo10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=("84848",),
                                                 phy_addrs=phy_addrs,
                                                 phy_layer='10GBaseT',
                                                 leds=leds,
                                                 mdi_pair_map=mdi_pair_map,
                                                 unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in range(self.num_physical_ports(mode)):
            set_calls[self.leds[1] + led] = call

        return set_calls

class QuantaLY9RangeleyQSFPp(cumulus.platform.TridentTwo40GPort):
    _set_led = '''
;;; QuantaLY9RangeleyQSFPp
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed is always zero
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
;;; QuantaLY9Rangeley10GT
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed should always be zero;
;;;
;;; 2nd bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
set_led:
        port 1
        pushst PORTSTATUS_ZERO
        pack

        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        push    PORTSTATUS_ZERO
        pack
        push    PORTSTATUS_ZERO ; 0, 0 => led 2 off
        pack
        push    PORTSTATUS_ZERO
        pack
        push    PORTSTATUS_ZERO ; 0, 0 => led 3 off
        pack
        push    PORTSTATUS_ZERO
        pack
        push    PORTSTATUS_ZERO ; 0, 0 => led 4 off
        pack

        ret

'''

    _set_led_4x10 = '''
;;; QuantaLY9RangeleyQSFPp_4x10
;;
;;  For 4x10G mode, we use an LED for each port.
;;
set_any_led:
        port 1
        pushst PORTSTATUS_ZERO
        pack

        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret

set_led:
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led
        inc     a
        call    set_any_led

        ret

'''
    def __init__(self, label, serdes, leds, phy_addrs, prim_offset,
                 phy_id1, sys_interface, phy_types):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
        cumulus.platform.TridentTwo40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_addrs=phy_addrs,
                                                 phy_layer=sfp.type.upper(),
                                                 phy_types=phy_types,
                                                 prim_offset=prim_offset,
                                                 phy_id1=phy_id1,
                                                 sys_interface=sys_interface,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = sfp
        if (self.label == "53") or (self.label == "54"):
            self._serdes_intf_map = {40:'SR4'}

    @property
    def max_logical_ports(self):
        if (self.label == "53" or self.label == "54"):
            return 1
        else:
            return 4

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None


        return set_calls

class QuantaLY9RangeleyLED(cumulus.ledup.TridentTwoLEDController):
    num_leds = 128

class QuantaLY9RangeleySwitch(cumulus.platform.BCMSwitch):
    ledup = QuantaLY9RangeleyLED
    _10GT = QuantaLY9Rangeley10GT
    _qsfp = QuantaLY9RangeleyQSFPp
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = True

    ports = (
        _10GT(label="1",   serdes=(25,),  phy_addrs=(0x22,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="2",   serdes=(24,),  phy_addrs=(0x21,),  leds=(0, 0), mdi_pair_map=0x3210),
        _10GT(label="3",   serdes=(27,),  phy_addrs=(0x24,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="4",   serdes=(26,),  phy_addrs=(0x23,),  leds=(0, 0), mdi_pair_map=0x3210),

        _10GT(label="5",   serdes=(29,),  phy_addrs=(0x26,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="6",   serdes=(28,),  phy_addrs=(0x25,),  leds=(0, 0), mdi_pair_map=0x3210),
        _10GT(label="7",   serdes=(31,),  phy_addrs=(0x28,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="8",   serdes=(30,),  phy_addrs=(0x27,),  leds=(0, 0), mdi_pair_map=0x3210),

        _10GT(label="9",   serdes=(45,),  phy_addrs=(0x2a,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="10",  serdes=(44,),  phy_addrs=(0x29,),  leds=(0, 0), mdi_pair_map=0x3210),
        _10GT(label="11",  serdes=(47,),  phy_addrs=(0x2c,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="12",  serdes=(46,),  phy_addrs=(0x2b,),  leds=(0, 0), mdi_pair_map=0x3210),

        _10GT(label="13",  serdes=(49,),  phy_addrs=(0x2e,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="14",  serdes=(48,),  phy_addrs=(0x2d,),  leds=(0, 0), mdi_pair_map=0x3210),
        _10GT(label="15",  serdes=(51,),  phy_addrs=(0x30,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="16",  serdes=(50,),  phy_addrs=(0x2f,),  leds=(0, 0), mdi_pair_map=0x3210),

        _10GT(label="17",  serdes=(53,),  phy_addrs=(0x32,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="18",  serdes=(52,),  phy_addrs=(0x31,),  leds=(0, 0), mdi_pair_map=0x3210),
        _10GT(label="19",  serdes=(55,),  phy_addrs=(0x34,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="20",  serdes=(54,),  phy_addrs=(0x33,),  leds=(0, 0), mdi_pair_map=0x3210),

        _10GT(label="21",  serdes=(57,),  phy_addrs=(0x36,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="22",  serdes=(56,),  phy_addrs=(0x35,),  leds=(0, 0), mdi_pair_map=0x3210),
        _10GT(label="23",  serdes=(59,),  phy_addrs=(0x38,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="24",  serdes=(58,),  phy_addrs=(0x37,),  leds=(0, 0), mdi_pair_map=0x3210),

        _10GT(label="25",  serdes=(61,),  phy_addrs=(0x42,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="26",  serdes=(60,),  phy_addrs=(0x41,),  leds=(0, 0), mdi_pair_map=0x3210),
        _10GT(label="27",  serdes=(63,),  phy_addrs=(0x44,),  leds=(0, 0), mdi_pair_map=0x0123),
        _10GT(label="28",  serdes=(62,),  phy_addrs=(0x43,),  leds=(0, 0), mdi_pair_map=0x3210),

        _10GT(label="29",  serdes=(65,),  phy_addrs=(0x46,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="30",  serdes=(64,),  phy_addrs=(0x45,),  leds=(1, 0), mdi_pair_map=0x3210),
        _10GT(label="31",  serdes=(67,),  phy_addrs=(0x48,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="32",  serdes=(66,),  phy_addrs=(0x47,),  leds=(1, 0), mdi_pair_map=0x3210),

        _10GT(label="33",  serdes=(69,),  phy_addrs=(0x4a,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="34",  serdes=(68,),  phy_addrs=(0x49,),  leds=(1, 0), mdi_pair_map=0x3210),
        _10GT(label="35",  serdes=(71,),  phy_addrs=(0x4c,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="36",  serdes=(70,),  phy_addrs=(0x4b,),  leds=(1, 0), mdi_pair_map=0x3210),

        _10GT(label="37",  serdes=(73,),  phy_addrs=(0x4e,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="38",  serdes=(72,),  phy_addrs=(0x4d,),  leds=(1, 0), mdi_pair_map=0x3210),
        _10GT(label="39",  serdes=(75,),  phy_addrs=(0x50,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="40",  serdes=(74,),  phy_addrs=(0x4f,),  leds=(1, 0), mdi_pair_map=0x3210),

        _10GT(label="41",  serdes=(77,),  phy_addrs=(0x52,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="42",  serdes=(76,),  phy_addrs=(0x51,),  leds=(1, 0), mdi_pair_map=0x3210),
        _10GT(label="43",  serdes=(79,),  phy_addrs=(0x54,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="44",  serdes=(78,),  phy_addrs=(0x53,),  leds=(1, 0), mdi_pair_map=0x3210),

        _10GT(label="45",  serdes=(81,),  phy_addrs=(0x56,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="46",  serdes=(80,),  phy_addrs=(0x55,),  leds=(1, 0), mdi_pair_map=0x3210),
        _10GT(label="47",  serdes=(83,),  phy_addrs=(0x58,),  leds=(1, 0), mdi_pair_map=0x0123),
        _10GT(label="48",  serdes=(82,),  phy_addrs=(0x57,),  leds=(1, 0), mdi_pair_map=0x3210),

        _qsfp(label="49", serdes=(96, 97, 98, 99), leds=(1, 19),
              phy_addrs=None, prim_offset=None, phy_id1=None, sys_interface=None, phy_types=None),
        _qsfp(label="50", serdes=(100, 101, 102, 103), leds=(1, 23),
              phy_addrs=None, prim_offset=None, phy_id1=None, sys_interface=None, phy_types=None),
        _qsfp(label="51", serdes=(104, 105, 106, 107), leds=(1, 27),
              phy_addrs=None, prim_offset=None, phy_id1=None, sys_interface=None, phy_types=None),
        _qsfp(label="52", serdes=(108, 109, 110, 111), leds=(1, 31),
              phy_addrs=None, prim_offset=None, phy_id1=None, sys_interface=None, phy_types=None),
        _qsfp(label="53", serdes=(20, 21, 22, 23), leds=(0, 107),
              phy_addrs = (0x14, 0x15, 0x16, 0x17), prim_offset=0x3600, phy_id1=0x8500, sys_interface=9,
              phy_types = ("84328",) * 4),
        _qsfp(label="54", serdes=(12, 13, 14, 15), leds=(0, 123),
              phy_addrs = (0x10, 0x11, 0x12, 0x13), prim_offset=0x3500, phy_id1=0x8500, sys_interface=9,
              phy_types = ("84328",) * 4),
    )

class QuantaLY9Rangeley(cumulus.platform.Platform):
    name = 'quanta,ly9_rangeley'
    chip = cumulus.platform.TridentTwo_56854_Chip()
    switch = QuantaLY9RangeleySwitch(chip)
    pwms = QuantaLY9RangeleyPWMs()
    sensors = QuantaLY9RangeleySensors()
    statusleds = QuantaLY9RangeleyLEDs()

Platform = QuantaLY9Rangeley
